package org.framework.lazy.cloud.network.heartbeat.protocol.handler;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.http.FullHttpRequest;
import lombok.extern.slf4j.Slf4j;
import org.framework.lazy.cloud.network.heartbeat.common.adapter.ChannelTypeAdapter;
import org.framework.lazy.cloud.network.heartbeat.common.decoder.TransferDecoder;
import org.framework.lazy.cloud.network.heartbeat.common.encoder.TransferEncoder;
import org.framework.lazy.cloud.network.heartbeat.common.utils.ChannelAttributeKeyUtils;
import org.framework.lazy.cloud.network.heartbeat.protocol.utils.FullHttpRequestUtils;

import java.net.URI;

/**
 * description 服务端数据处理器
 *
 * @author 吴佳伟
 * @date 2023/09/13 10:27
 */
@Slf4j
public class NettyHttpProxyHandler extends ChannelInboundHandlerAdapter {

    private final ChannelTypeAdapter channelTypeAdapter;


    public NettyHttpProxyHandler(ChannelTypeAdapter channelTypeAdapter) {
        this.channelTypeAdapter = channelTypeAdapter;
    }

    private Channel outboundChannel;

    @Override
    public void channelRead(final ChannelHandlerContext ctx, Object msg) throws Exception {
        if (msg instanceof FullHttpRequest) {
            FullHttpRequest request = (FullHttpRequest) msg;
            URI uri = new URI(request.uri());
            String host = uri.getHost();
            int port = uri.getPort();
            if (port == -1) {
                port = 80;
            }

            Bootstrap b = new Bootstrap();
            b.group(ctx.channel().eventLoop())
                    .channel(NioSocketChannel.class)
                    .handler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel ch) throws Exception {
                            ch.pipeline().addLast(
//                                    new HttpClientCodec(),
//                                    new HttpRequestDecoder(),
//                                    new HttpObjectAggregator(1048576),
                                    new TransferDecoder(Integer.MAX_VALUE, 1024 * 1024 * 10),
                                    new TransferEncoder(),
                                    new NettyHttpProxyBackendHandler(ctx.channel())
                            );
                        }
                    });
            ChannelFuture f = b.connect(host, port);
            outboundChannel = f.channel();
            f.addListener((ChannelFutureListener) future -> {
                if (future.isSuccess()) {
                    ChannelAttributeKeyUtils.buildNextChannel(outboundChannel, ctx.channel());
                    ChannelAttributeKeyUtils.buildNextChannel(ctx.channel(), outboundChannel);
                    outboundChannel.writeAndFlush(FullHttpRequestUtils.toByteBuf(request));
                } else {
                    ctx.channel().close();
                }
            });
        } else {
            log.warn("this is not http request");
        }
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        if (outboundChannel != null) {
            closeOnFlush(outboundChannel);
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        closeOnFlush(ctx.channel());
    }

    static void closeOnFlush(Channel ch) {
        if (ch.isActive()) {
            ch.writeAndFlush(Unpooled.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
        }
    }
}